from random import randint
from typing import ClassVar
from unittest import TestCase
from unittest.mock import patch


class Dragon:
    HEALTH_MIN: ClassVar[int] = 50
    HEALTH_MAX: ClassVar[int] = 100
    name: str
    health: int
    position_x: int
    position_y: int

    def __init__(self, name: str, /,
                 *, position_x: int = 0, position_y: int = 0,
                 ) -> None:
        self.name = name
        self.health = randint(self.HEALTH_MIN, self.HEALTH_MAX)
        self.position_x = position_x
        self.position_y = position_y


class TestName(TestCase):
    def test_name_default(self):
        with self.assertRaises(TypeError):
            dragon = Dragon()  # noqa

    def test_name_positional(self):
        dragon = Dragon('MyName')
        self.assertEqual(dragon.name, 'MyName')

    def test_name_keyword(self):
        with self.assertRaises(TypeError):
            dragon = Dragon(name='MyName')  # noqa


class TestHealth(TestCase):
    def setUp(self):
        self.dragon = Dragon('MyName')

    def test_health_default(self):
        self.assertGreaterEqual(self.dragon.health, 50)
        self.assertLessEqual(self.dragon.health, 100)

    def test_health_default_many(self):
        for _ in range(10_000):
            self.dragon = Dragon('MyName')
            self.assertGreaterEqual(self.dragon.health, 50)
            self.assertLessEqual(self.dragon.health, 100)

    def test_health_default_patch(self):
        with patch('dragon.randint', return_value=74) as randint:  # noqa
            dragon = Dragon('MyName')
            self.assertEqual(dragon.health, 74)

    def test_health_hasattr_HEALTH_MIN(self):
        self.assertTrue(hasattr(Dragon, 'HEALTH_MIN'))
        self.assertEqual(Dragon.HEALTH_MIN, 50)

    def test_health_hasattr_HEALTH_MAX(self):
        self.assertTrue(hasattr(Dragon, 'HEALTH_MAX'))
        self.assertEqual(Dragon.HEALTH_MAX, 100)


class TestPosition(TestCase):
    def setUp(self):
        self.dragon = Dragon('MyName')

    def test_position_init_default(self):
        self.assertEqual(self.dragon.position_x, 0)
        self.assertEqual(self.dragon.position_y, 0)

    def test_position_init_positional(self):
        with self.assertRaises(TypeError):
            dragon = Dragon('MyName', 1, 2)  # noqa

    def test_position_init_keyword(self):
        dragon = Dragon('MyName', position_x=1, position_y=2)
        self.assertEqual(dragon.position_x, 1)
        self.assertEqual(dragon.position_y, 2)
